//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

/* ------------------------- */
/* --- Value                 */
/* ------------------------- */

/* - control target */
BYTE  g_TargetTemp;

/* - temp */
BYTE  g_Temp;

/* - heater enable */
BYTE  g_HeaterSw;

/* - Hotplate Controller Switch */
BYTE  g_MainSwitch;

/* - led blink */
BYTE  g_Blink;
BYTE  g_Counter1;

/* - key port with repeart */
BYTE  g_KeyInput;
BYTE  g_KeyBefore;
BYTE  g_Counter2;

/* - lowpass */
INT g_Filter;


/* ------------------------------------------------------ */

/* ------------------------- */
/* --- Define                */
/* ------------------------- */

#pragma abs_address 0x7FC0    //@EEPROM_1 
 const BYTE InitialTemp = 110; 
#pragma end_abs_address

/* --- Thermistor value trans table */
#define NUMTEMPS 21

short temptable[NUMTEMPS][2] = {
   {0, 841},
   {54, 255},
   {107, 209},
   {160, 184},
   {213, 166},
   {266, 153},
   {319, 142},
   {372, 132},
   {425, 124},
   {478, 116},
   {531, 108},
   {584, 101},
   {637, 93},
   {690, 86},
   {743, 78},
   {796, 70},
   {849, 61},
   {902, 50},
   {955, 34},
   {1008, 3},
   {1024, 0}
};

/* ------------------------------------------------------ */

/* ------------------------- */
/* --- Interrupt             */
/* ------------------------- */

#pragma interrupt_handler Timer1Int
void Timer1Int(void)
{
	g_Counter1 += 1;
	g_Blink ^= (g_Counter1 & 1);
}

#pragma interrupt_handler Timer10msInt
void Timer10msInt(void)
{
	BYTE port;
	port = PRT0DR;
	g_KeyInput = port ^ 0xff;
	if ((g_KeyInput & 0b00000101) == 0) {
		g_Counter2 = 0;
	}
	else {
		if (g_Counter2 != 0) {
		   if (g_Counter2 > 64) {
		      g_Counter2 = 50;
		   }
		   if (g_Counter2 != 50) {
		      g_KeyInput &= 0b00000010;
		   }
		}
		g_Counter2 += 1;
	}
}


/* ------------------------- */
/* --- Key Control */
void key_input(void)
{
	BYTE work;
	work = g_KeyInput & (g_KeyBefore ^ 0xff);
	if (work & 0b00000001) {
		if (g_TargetTemp > 30) {
			g_TargetTemp -= 1;
		}
	}
	if (work & 0b00000010) {
		g_MainSwitch ^= 1;
		if (g_MainSwitch == 0) {
			/* write temp to E2PROM */
			E2PROM_1_bE2Write(0, &g_TargetTemp, 1, 25);
		}
	}
	if (work & 0b00000100) {
		if (g_TargetTemp < 140) {
			g_TargetTemp += 1;
		}
	}
	g_KeyBefore = g_KeyInput;
}

/* ------------------------- */
/* --- Key LED */
void key_led(void)
{
	BYTE pin_in, pin_out;
	pin_in = PRT0DR;
	pin_out = 0;
	/* main sw */
	if (g_MainSwitch) {
		pin_out |= 0b00010000;
		/* temp low */
		if ((g_TargetTemp > g_Temp) && (g_Blink != 0) && ((g_TargetTemp - g_Temp) > 2)) {
			pin_out |= 0b00001000;
		}
		/* temp high */
		if ((g_TargetTemp < g_Temp) && (g_Blink != 0) && ((g_Temp - g_TargetTemp) > 2)) {
			pin_out |= 0b00100000;
		}
	}
	/* led set */
	pin_out = (pin_in & 0b11000111) | pin_out;
	PRT0DR = pin_out;
}

/* ------------------------- */
/* --- Display */
void print_dec_byte(BYTE value)
{
	BYTE dispflag, ch;
	dispflag = 0; 
	ch = '0';
	while(value >= 100) {
	  dispflag = 1;
	  value -= 100;
	  ch += 1;
	}
	if (dispflag) {
		LCD_1_WriteData(ch);
	}
	else {
		LCD_1_WriteData(' ');
	}
	ch = '0';
	while(value >= 10) {
	  dispflag = 1;
	  value -= 10;
	  ch += 1;
	}
	if (dispflag) {
		LCD_1_WriteData(ch);
	}
	else {
		LCD_1_WriteData(' ');
	}
	ch = '0' + value;
	LCD_1_WriteData(ch);
}

void do_display(void)
{
	BYTE  anime;

	/* --- Display */
	LCD_1_Position(0, 0);
	LCD_1_PrCString("Temp = ");
	print_dec_byte(g_Temp);
	//LCD_1_Position(0, 8);
	LCD_1_WriteData('[');
	//LCD_1_Position(0, 9);
	print_dec_byte(g_TargetTemp);
	//LCD_1_Position(0, 12);
	LCD_1_WriteData(']');
	
	if (g_MainSwitch) {
		LCD_1_Position(1, 0);
		LCD_1_PrCString(" [ ON ] ");
		if (g_HeaterSw) {
			anime = g_Counter1 & 3;
			switch(anime) {
			  case 0:
				LCD_1_PrCString("   *   ");
				break;
			  case 1:
				LCD_1_PrCString("  (*)  ");
				break;
			  case 2:
				LCD_1_PrCString(" ((*)) ");
				break;
			  case 3:
				LCD_1_PrCString("(((*)))");
				break;
			}
		}
		else {
			LCD_1_PrCString("       ");
		}
	}
	else {
		LCD_1_Position(1, 0);
		LCD_1_PrCString("   off         ");
	}
}

/* ------------------------- */
/* --- Heater Controller */
void get_temp(void)
{
	WORD  val, temp;
	short temp_band, area, pos;
	BYTE  i;
	if ( ADCINCVR_1_fIsDataAvailable() ) {
		val = ADCINCVR_1_iGetDataClearFlag();
		g_Filter = g_Filter + val;
		g_Filter /= 2;
		val = g_Filter;
		/* adc to temp */
		temp = 0;
		for(i=0; i<(NUMTEMPS-1); i++) {
			if ((temptable[i][0] <= val) && (temptable[i+1][0] > val)) {
				temp_band = temptable[i][1] - temptable[i+1][1];
				area = temptable[i+1][0] - temptable[i][0];
				pos = val - temptable[i][0];
				temp = temp_band * pos / area;
				temp = temptable[i][1] - temp;
				break;
			}
		}
		g_Temp = temp;
	}
}

void do_heatcontrol(void)
{
	BYTE pin_in, pin_out;

	if (g_HeaterSw) {
		if (g_Temp > (g_TargetTemp+2)) {
			g_HeaterSw = 0;
		}
	}
	else {
		if (g_Temp < (g_TargetTemp-2)) {
			g_HeaterSw = 1;
		}
	}
	g_HeaterSw &= g_MainSwitch;
	/* output */
	pin_in = PRT0DR;
	pin_out = 0;
	if (g_HeaterSw) {
		pin_out |= 0b01000000;
	}
	/* led set */
	pin_out = (pin_in & 0b10111111) | pin_out;
	PRT0DR = pin_out;
}

/* ------------------------- */
/* --- Main routine          */
/* ------------------------- */
void main(void)
{
	BYTE t;

	/* --- module initialize */
    LCD_1_Start();
	LCD_1_Init();
	PGA_1_Start(PGA_1_HIGHPOWER);
	ADCINCVR_1_Start(ADCINCVR_1_HIGHPOWER);
	Counter16_1_EnableInt();
	Counter_10ms_EnableInt();
	M8C_EnableGInt;
	Counter16_1_Start();
	Counter_10ms_Start();
	ADCINCVR_1_GetSamples(0);
	ADCINCVR_1_ClearFlag();
	E2PROM_1_Start();
	
	
	/* --- Initialize */
	g_TargetTemp = 110;
	g_Temp = 0;
	g_HeaterSw = 0;
	g_MainSwitch = 0;
	g_Counter1 = 0;
	g_Counter2 = 0;
	g_Blink = 0;
	g_KeyBefore = 0;
	t = g_Blink;
	
	/* --- read target from E2PROM */
	E2PROM_1_E2Read(0, &g_TargetTemp, 1);
	
	/* --- end */
	while(1) {
		key_input();
		key_led();
		do_heatcontrol();
		do_display();
		if (g_Blink != t) {
			get_temp();
			t = g_Blink;
		}
		LCD_1_Delay50uTimes(50);
	}
}
